==> sri-d3m-1.2.5.tar.gz: sri-d3m-1.2.5/sri/psl/collaborative_filtering_link_prediction.py <==
import typing

import pandas
from d3m import container
from d3m.metadata import base as meta_base
from d3m.metadata import hyperparams as meta_hyperparams
from d3m.metadata import params as meta_params
from d3m.primitive_interfaces import base as pi_base
from d3m.primitive_interfaces import unsupervised_learning as pi_unsupervised_learning

from sri.common import config
from sri.common import constants
from sri.common import util
from sri.graph.collaborative_filtering import CollaborativeFilteringParser
from sri.graph.collaborative_filtering import CollaborativeFilteringParserHyperparams
from sri.graph.transform import GraphTransformerHyperparams
from sri.graph.transform import GraphTransformer
from sri.graph.networkx import Graph
from sri.psl.link_prediction import LinkPrediction
from sri.psl.link_prediction import LinkPredictionHyperparams

# The test data will just look like a dataset, but the match column will be empty.
Inputs = container.Dataset
# Return match or no match.
Outputs = container.DataFrame

class CollaborativeFilteringLinkPredictionHyperparams(meta_hyperparams.Hyperparams):
    scale_rating = meta_hyperparams.Hyperparameter[bool](
            default = True,
            semantic_types = ['https://metadata.datadrivendiscovery.org/types/ControlParameter'])

    collaborative_filtering_parser_hyperparams = meta_hyperparams.Hyperparameter(
            default = CollaborativeFilteringParserHyperparams.defaults(),
            semantic_types = ['https://metadata.datadrivendiscovery.org/types/ControlParameter'])

    graph_transform_hyperparams = meta_hyperparams.Hyperparameter(
            default = GraphTransformerHyperparams.defaults(),
            semantic_types = ['https://metadata.datadrivendiscovery.org/types/ControlParameter'])

    link_prediction_hyperparams = meta_hyperparams.Hyperparameter(
            default = LinkPredictionHyperparams.defaults(),
            semantic_types = ['https://metadata.datadrivendiscovery.org/types/ControlParameter'])

class CollaborativeFilteringLinkPredictionParams(meta_params.Params):
    training_set: Inputs
    debug_options: typing.Dict

class CollaborativeFilteringLinkPrediction(pi_unsupervised_learning.UnsupervisedLearnerPrimitiveBase[Inputs, Outputs, CollaborativeFilteringLinkPredictionParams, CollaborativeFilteringLinkPredictionHyperparams]):
    """
    A primitive that takes a collaborative filtering problem and does full link prediction on it.
    This just strings together various other primitives.

    Note: this is unsupervised because we take the dataset itself and set_training_data() does not want an output argument.
    """

    def __init__(self, *, hyperparams: CollaborativeFilteringLinkPredictionHyperparams, random_seed: int = 0, _debug_options: typing.Dict = {}) -> None:
        super().__init__(hyperparams = hyperparams, random_seed = random_seed)

        self._cf_parser = CollaborativeFilteringParser(
                hyperparams = self.hyperparams['collaborative_filtering_parser_hyperparams'],
                _debug_options = _debug_options)

        self._transformer = GraphTransformer(
                hyperparams = self.hyperparams['graph_transform_hyperparams'],
                _debug_options = _debug_options)

        self._link_prediction = LinkPrediction(
                hyperparams = self.hyperparams['link_prediction_hyperparams'],
                _debug_options = _debug_options)

        self._training_dataset = None
        self._set_debug_options(_debug_options)

    def _set_debug_options(self, _debug_options):
        self._debug_options = _debug_options

        self._cf_parser._set_debug_options(_debug_options)
        self._transformer._set_debug_options(_debug_options)
        self._link_prediction._set_debug_options(_debug_options)

    def set_training_data(self, *, inputs: Inputs) -> None:
        self._training_dataset = inputs

    def fit(self, *, timeout: float = None, iterations: int = None) -> pi_base.CallResult[None]:
        return pi_base.CallResult(None)

    def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> pi_base.CallResult[Outputs]:
        d3m_indexes = self._parse_d3m_indexes(inputs)
        full_data = self._concat_datasets(self._training_dataset, inputs)

        parsed_graphs = self._cf_parser.produce(inputs = full_data).value
        input_graph = self._transformer.produce(inputs = parsed_graphs).value

        results = self._link_prediction.produce(inputs = input_graph, timeout = timeout, iterations = iterations).value

        # Modify the results.

        # Rename the 'link' column to 'rating'.
        results.rename(columns = {'link': 'rating'}, inplace = True)

        # Move the confidence over to the rating, possibly rescaling.
        min_rating = self.hyperparams['collaborative_filtering_parser_hyperparams']['min_rating']
        max_rating = self.hyperparams['collaborative_filtering_parser_hyperparams']['max_rating']
        if (self.hyperparams['scale_rating']):
            results['rating'] = results[constants.CONFIDENCE_COLUMN] * (max_rating - min_rating) + min_rating
        else:
            results['rating'] = results[constants.CONFIDENCE_COLUMN]

        # Drop the confidence column (this column is allowed in the standard, but few people use it).
        results = results.drop(columns = [constants.CONFIDENCE_COLUMN], errors = 'ignore')

        missing_value = (max_rating - min_rating) / 2.0
        results = util.prep_predictions(results, d3m_indexes, metadata_source = self, missing_value = missing_value)

        return pi_base.CallResult(results)

    def _parse_d3m_indexes(self, dataset):
        return list(dataset['0'][constants.D3M_INDEX])

    def _concat_datasets(self, dataset1, dataset2):
        new_data = pandas.concat([dataset1['0'], dataset2['0']], ignore_index = True)

        resources = {'0': new_data}
        metadata = dataset1.metadata.update(('0',), {'dimension': {'length': len(new_data)}})

        metadata.check(resources)
        return container.Dataset(resources, metadata)

    def get_params(self) -> CollaborativeFilteringLinkPredictionParams:
        return CollaborativeFilteringLinkPredictionParams({
            'training_set': self._training_dataset,
            'debug_options': self._debug_options
        })

    def set_params(self, *, params: CollaborativeFilteringLinkPredictionParams) -> None:
        self._training_dataset = params['training_set']
        self._set_debug_options(params['debug_options'])

    metadata = meta_base.PrimitiveMetadata({
        # Required
        'id': 'f9440843-5791-4d58-b0a8-617b5cb2371d',
        'version': config.VERSION,
        'name': 'Collaborative Filtering Link Prediction',
        'description': 'Give a full solution to "collaborative filtering"-like problems using collective link prediction.',
        'python_path': 'd3m.primitives.link_prediction.collaborative_filtering_link_prediction.CollaborativeFilteringLinkPrediction',
        'primitive_family': meta_base.PrimitiveFamily.LINK_PREDICTION,
        'algorithm_types': [
            meta_base.PrimitiveAlgorithmType.MARKOV_RANDOM_FIELD,
        ],
        'source': config.SOURCE,

        # Optional
        'keywords': [ 'preprocessing', 'primitive', 'graph', 'dataset', 'transformer', 'link prediction', 'collective classifiction', 'collaborativeFiltering' ],
        'installation': [ config.INSTALLATION ],
        'location_uris': [],
        'preconditions': [ meta_base.PrimitiveEffect.NO_MISSING_VALUES ],
        'effects': [ meta_base.PrimitiveEffect.NO_MISSING_VALUES ],
        'hyperparms_to_tune': []
    })

==> sri-d3m-1.3.0.tar.gz: sri-d3m-1.3.0/sri/psl/collaborative_filtering_link_prediction.py <==
import typing

import pandas
from d3m import container
from d3m.metadata import base as meta_base
from d3m.metadata import hyperparams as meta_hyperparams
from d3m.metadata import params as meta_params
from d3m.primitive_interfaces import base as pi_base
from d3m.primitive_interfaces import unsupervised_learning as pi_unsupervised_learning

from sri.common import config
from sri.common import constants
from sri.common import util
from sri.graph.collaborative_filtering import CollaborativeFilteringParser
from sri.graph.collaborative_filtering import CollaborativeFilteringParserHyperparams
from sri.graph.transform import GraphTransformerHyperparams
from sri.graph.transform import GraphTransformer
from sri.graph.networkx import Graph
from sri.psl.link_prediction import LinkPrediction
from sri.psl.link_prediction import LinkPredictionHyperparams

# The test data will just look like a dataset, but the match column will be empty.
Inputs = container.Dataset
# Return match or no match.
Outputs = container.DataFrame

class CollaborativeFilteringLinkPredictionHyperparams(meta_hyperparams.Hyperparams):
    scale_rating = meta_hyperparams.Hyperparameter[bool](
            default = True,
            semantic_types = ['https://metadata.datadrivendiscovery.org/types/ControlParameter'])

    collaborative_filtering_parser_hyperparams = meta_hyperparams.Hyperparameter(
            default = CollaborativeFilteringParserHyperparams.defaults(),
            semantic_types = ['https://metadata.datadrivendiscovery.org/types/ControlParameter'])

    graph_transform_hyperparams = meta_hyperparams.Hyperparameter(
            default = GraphTransformerHyperparams.defaults(),
            semantic_types = ['https://metadata.datadrivendiscovery.org/types/ControlParameter'])

    link_prediction_hyperparams = meta_hyperparams.Hyperparameter(
            default = LinkPredictionHyperparams.defaults(),
            semantic_types = ['https://metadata.datadrivendiscovery.org/types/ControlParameter'])

class CollaborativeFilteringLinkPredictionParams(meta_params.Params):
    training_set: Inputs
    debug_options: typing.Dict

class CollaborativeFilteringLinkPrediction(pi_unsupervised_learning.UnsupervisedLearnerPrimitiveBase[Inputs, Outputs, CollaborativeFilteringLinkPredictionParams, CollaborativeFilteringLinkPredictionHyperparams]):
    """
    A primitive that takes a collaborative filtering problem and does full link prediction on it.
    This just strings together various other primitives.

    Note: this is unsupervised because we take the dataset itself and set_training_data() does not want an output argument.
    """

    def __init__(self, *, hyperparams: CollaborativeFilteringLinkPredictionHyperparams, random_seed: int = 0, _debug_options: typing.Dict = {}) -> None:
        super().__init__(hyperparams = hyperparams, random_seed = random_seed)

        self._cf_parser = CollaborativeFilteringParser(
                hyperparams = self.hyperparams['collaborative_filtering_parser_hyperparams'],
                _debug_options = _debug_options)

        self._transformer = GraphTransformer(
                hyperparams = self.hyperparams['graph_transform_hyperparams'],
                _debug_options = _debug_options)

        self._link_prediction = LinkPrediction(
                hyperparams = self.hyperparams['link_prediction_hyperparams'],
                _debug_options = _debug_options)

        self._training_dataset = None
        self._set_debug_options(_debug_options)

    def _set_debug_options(self, _debug_options):
        self._debug_options = _debug_options

        self._cf_parser._set_debug_options(_debug_options)
        self._transformer._set_debug_options(_debug_options)
        self._link_prediction._set_debug_options(_debug_options)

    def set_training_data(self, *, inputs: Inputs) -> None:
        self._training_dataset = inputs

    def fit(self, *, timeout: float = None, iterations: int = None) -> pi_base.CallResult[None]:
        return pi_base.CallResult(None)

    def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> pi_base.CallResult[Outputs]:
        d3m_indexes = self._parse_d3m_indexes(inputs)
        full_data = self._concat_datasets(self._training_dataset, inputs)

        parsed_graphs = self._cf_parser.produce(inputs = full_data).value
        input_graph = self._transformer.produce(inputs = parsed_graphs).value

        results = self._link_prediction.produce(inputs = input_graph, timeout = timeout, iterations = iterations).value

        # Modify the results.

        # Rename the 'link' column to 'rating'.
        results.rename(columns = {'link': 'rating'}, inplace = True)

        # Move the confidence over to the rating, possibly rescaling.
        min_rating = self.hyperparams['collaborative_filtering_parser_hyperparams']['min_rating']
        max_rating = self.hyperparams['collaborative_filtering_parser_hyperparams']['max_rating']
        if (self.hyperparams['scale_rating']):
            results['rating'] = results[constants.CONFIDENCE_COLUMN] * (max_rating - min_rating) + min_rating
        else:
            results['rating'] = results[constants.CONFIDENCE_COLUMN]

        # Drop the confidence column (this column is allowed in the standard, but few people use it).
        results = results.drop(columns = [constants.CONFIDENCE_COLUMN], errors = 'ignore')

        missing_value = (max_rating - min_rating) / 2.0
        results = util.prep_predictions(results, d3m_indexes, metadata_source = self, missing_value = missing_value)

        return pi_base.CallResult(results)

    def _parse_d3m_indexes(self, dataset):
        return list(dataset[constants.D3M_TABLE_KEY][constants.D3M_INDEX])

    def _concat_datasets(self, dataset1, dataset2):
        new_data = pandas.concat([dataset1[constants.D3M_TABLE_KEY], dataset2[constants.D3M_TABLE_KEY]], ignore_index = True)

        resources = {constants.D3M_TABLE_KEY: new_data}
        metadata = dataset1.metadata.update((constants.D3M_TABLE_KEY,), {'dimension': {'length': len(new_data)}})

        metadata.check(resources)
        return container.Dataset(resources, metadata, generate_metadata=True)

    def get_params(self) -> CollaborativeFilteringLinkPredictionParams:
        return CollaborativeFilteringLinkPredictionParams({
            'training_set': self._training_dataset,
            'debug_options': self._debug_options
        })

    def set_params(self, *, params: CollaborativeFilteringLinkPredictionParams) -> None:
        self._training_dataset = params['training_set']
        self._set_debug_options(params['debug_options'])

    metadata = meta_base.PrimitiveMetadata({
        # Required
        'id': 'f9440843-5791-4d58-b0a8-617b5cb2371d',
        'version': config.VERSION,
        'name': 'Collaborative Filtering Link Prediction',
        'description': 'Give a full solution to "collaborative filtering"-like problems using collective link prediction.',
        'python_path': 'd3m.primitives.link_prediction.collaborative_filtering_link_prediction.CollaborativeFilteringLinkPrediction',
        'primitive_family': meta_base.PrimitiveFamily.LINK_PREDICTION,
        'algorithm_types': [
            meta_base.PrimitiveAlgorithmType.MARKOV_RANDOM_FIELD,
        ],
        'source': config.SOURCE,

        # Optional
        'keywords': [ 'preprocessing', 'primitive', 'graph', 'dataset', 'transformer', 'link prediction', 'collective classifiction', 'collaborativeFiltering' ],
        'installation': [ config.INSTALLATION ],
        'location_uris': [],
        'preconditions': [ meta_base.PrimitiveEffect.NO_MISSING_VALUES ],
        'effects': [ meta_base.PrimitiveEffect.NO_MISSING_VALUES ],
        'hyperparms_to_tune': []
    })
